package com.novogroup.lotr.sms;



import java.io.FileInputStream;

import java.io.IOException;

import java.io.InputStream;

import java.io.PrintStream;

import java.io.PrintWriter;

import java.net.HttpURLConnection;

import java.net.URL;

import java.util.Iterator;

import java.util.Set;



import javax.naming.InitialContext;

import javax.servlet.ServletException;

import javax.servlet.http.HttpServlet;

import javax.servlet.http.HttpServletRequest;

import javax.servlet.http.HttpServletResponse;



import org.apache.commons.logging.Log;

import org.apache.commons.logging.LogFactory;



import com.novogroup.lotr.exception.SMSException;



import weblogic.management.Helper;

import weblogic.management.MBeanHome;

import weblogic.management.WebLogicMBean;

import weblogic.management.configuration.SSLMBean;

import weblogic.net.http.HttpsURLConnection;

import weblogic.security.SSL.SSLContext;





/**

 * @author Petri Hakala

 *

 */

public class SSLTestServlet extends HttpServlet {



    Log logger = LogFactory.getLog(this.getClass());



    public void doPost(HttpServletRequest req, HttpServletResponse res)

        throws ServletException

    {

        PrintStream ps = null;

        PrintWriter out = null;



        try

        {

            String body = req.getParameter("body");



            int contentLength = (body == null) ? 0 : body.length();



            logger.debug("contentLength :: " + contentLength);



            HttpURLConnection connection = openHTTPConnection(req);



            connection.setRequestMethod("POST");

            connection.setDoOutput(true);



            if (body != null)

            {

                ps = new PrintStream(connection.getOutputStream());

                ps.print(body);

            }



            int responseStatus = connection.getResponseCode();



            out = res.getWriter();

            out.println("SMSReceiver returned status code: " + responseStatus);

        }

        catch (Exception ioe)

        {

            logger.error("Error with output stream: ", ioe);

        }

        finally

        {

            if (ps != null)

            {

                ps.close();

            }

            if (out != null)

            {

                out.close();

            }

        }

    }



    /**

     * <p>Creats and shows a simple GUI for creating "SMS messages" to user</p>

     *

     * @param req        The request into the servlet.

     * @param res        The response to be sent back to caller.

     *

     * @throws           ServletException when initialization fails.

     */

    public void doGet(HttpServletRequest req, HttpServletResponse res)

        throws ServletException

    {

        PrintWriter out = null;



        try

        {

            out = res.getWriter();



            out.println("<html><head><title>SSL test</title><meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\"></head>");

            out.println("<body bgcolor=\"#FFFFFF\">");

            out.println("<form method=\"post\">");

            out.println("<table width=\"50%\" border=\"0\">");

            out.println("<tr><td width=\"21%\">TEXT:</td><td width=\"79%\"><textarea name=\"body\"></textarea></td></tr></table>");

            out.println("<p><input type=\"submit\" name=\"Submit\" value=\"Send\"></p></form></body></html>");

        }

        catch (IOException ioe)

        {

            logger.error("Error with output stream: " + ioe);

        }

        finally

        {

            if (out != null)

            {

                out.close();

            }

        }

    }



    /*

     * Opens HTTP connection to Shoppers SMS receiver servlet.

     */

    private HttpURLConnection openHTTPConnection(HttpServletRequest request) throws Exception

    {

        try

        {

            String url = null;

            url = "https://" + request.getServerName() + ":7002/backend/sms";

            InitialContext initial = new InitialContext();

            MBeanHome home = (MBeanHome)initial.lookup("weblogic.management.home.localhome");

            Set beans = home.getMBeansByType("SSLConfig");

            SSLMBean ssl = null;

            for (Iterator iter = beans.iterator(); iter.hasNext();) {

                ssl = (SSLMBean)iter.next();

                logger.debug("server: " + ssl.getServerCertificateFileName());

                logger.debug("trust: " + ssl.getTrustedCAFileName());

                logger.debug("key: " + ssl.getServerKeyFileName());

                break;

            }

            if(ssl == null) {

                throw new SMSException("Can't get SSL configuration");

            }

            URL smsSenderUrl = new URL(url);

            HttpsURLConnection connection = (HttpsURLConnection) smsSenderUrl.openConnection();

            SSLContext ctx = SSLContext.getInstance("https");

            FileInputStream in1 = new FileInputStream(ssl.getServerKeyFileName());

            FileInputStream in2 = new FileInputStream(ssl.getServerCertificateFileName());

            FileInputStream in3 = new FileInputStream(ssl.getTrustedCAFileName());

            InputStream in[] = new InputStream[]{ in1, in2, in3 };

            ctx.loadLocalIdentity(in);

            connection.setSSLSocketFactory(ctx.getSocketFactory());

            return connection;

        }

        catch (Exception e)

        {

            logger.error("Error opening URLConnection: " + e);

            throw e;

        }

    }

    

    



}

